/**
 * Copyright (c) 2014-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

 // @Generated by gentest/gentest.rb from gentest/fixtures/CSSLayoutRoundingTest.html

#include <CSSLayout/CSSLayout.h>
#include <gtest/gtest.h>

TEST(CSSLayoutTest, rounding_flex_basis_flex_grow_row_width_of_100) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetFlexDirection(root, CSSFlexDirectionRow);
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 100);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(34, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(67, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(67, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(34, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(33, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_flex_basis_flex_grow_row_prime_number_width) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetFlexDirection(root, CSSFlexDirectionRow);
  CSSNodeStyleSetWidth(root, 113);
  CSSNodeStyleSetHeight(root, 100);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeInsertChild(root, root_child2, 2);

  const CSSNodeRef root_child3 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child3, 1);
  CSSNodeInsertChild(root, root_child3, 3);

  const CSSNodeRef root_child4 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child4, 1);
  CSSNodeInsertChild(root, root_child4, 4);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(22, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(45, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  ASSERT_FLOAT_EQ(68, CSSNodeLayoutGetLeft(root_child3));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child3));
  ASSERT_FLOAT_EQ(22, CSSNodeLayoutGetWidth(root_child3));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child3));

  ASSERT_FLOAT_EQ(90, CSSNodeLayoutGetLeft(root_child4));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child4));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child4));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child4));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(90, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(68, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(22, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(45, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetLeft(root_child3));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child3));
  ASSERT_FLOAT_EQ(22, CSSNodeLayoutGetWidth(root_child3));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child3));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child4));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child4));
  ASSERT_FLOAT_EQ(23, CSSNodeLayoutGetWidth(root_child4));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child4));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_flex_basis_flex_shrink_row) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetFlexDirection(root, CSSFlexDirectionRow);
  CSSNodeStyleSetWidth(root, 101);
  CSSNodeStyleSetHeight(root, 100);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexShrink(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 100);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexBasis(root_child1, 25);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexBasis(root_child2, 25);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(101, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(51, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(51, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(76, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(101, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(50, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(51, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_flex_basis_overrides_main_size) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 113);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 50);
  CSSNodeStyleSetHeight(root_child0, 20);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeStyleSetHeight(root_child2, 10);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_total_fractial) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetWidth(root, 87.4f);
  CSSNodeStyleSetHeight(root, 113.4f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 0.7f);
  CSSNodeStyleSetFlexBasis(root_child0, 50.3f);
  CSSNodeStyleSetHeight(root_child0, 20.3f);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1.6f);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1.1f);
  CSSNodeStyleSetHeight(root_child2, 10.7f);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(30, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(30, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_total_fractial_nested) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetWidth(root, 87.4f);
  CSSNodeStyleSetHeight(root, 113.4f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 0.7f);
  CSSNodeStyleSetFlexBasis(root_child0, 50.3f);
  CSSNodeStyleSetHeight(root_child0, 20.3f);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child0_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0_child0, 0.3f);
  CSSNodeStyleSetPosition(root_child0_child0, CSSEdgeBottom, 13.3f);
  CSSNodeStyleSetHeight(root_child0_child0, 9.9f);
  CSSNodeInsertChild(root_child0, root_child0_child0, 0);

  const CSSNodeRef root_child0_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0_child1, 4);
  CSSNodeStyleSetFlexBasis(root_child0_child1, 0.3f);
  CSSNodeStyleSetPosition(root_child0_child1, CSSEdgeTop, 13.3f);
  CSSNodeStyleSetHeight(root_child0_child1, 1.1f);
  CSSNodeInsertChild(root_child0, root_child0_child1, 1);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1.6f);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1.1f);
  CSSNodeStyleSetHeight(root_child2, 10.7f);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0_child0));
  ASSERT_FLOAT_EQ(-13, CSSNodeLayoutGetTop(root_child0_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0_child0));
  ASSERT_FLOAT_EQ(12, CSSNodeLayoutGetHeight(root_child0_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetTop(root_child0_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0_child1));
  ASSERT_FLOAT_EQ(47, CSSNodeLayoutGetHeight(root_child0_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(30, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0_child0));
  ASSERT_FLOAT_EQ(-13, CSSNodeLayoutGetTop(root_child0_child0));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0_child0));
  ASSERT_FLOAT_EQ(12, CSSNodeLayoutGetHeight(root_child0_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetTop(root_child0_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child0_child1));
  ASSERT_FLOAT_EQ(47, CSSNodeLayoutGetHeight(root_child0_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(59, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(30, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(87, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_fractial_input_1) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 113.4f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 50);
  CSSNodeStyleSetHeight(root_child0, 20);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeStyleSetHeight(root_child2, 10);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_fractial_input_2) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 113.6f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 50);
  CSSNodeStyleSetHeight(root_child0, 20);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeStyleSetHeight(root_child2, 10);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(114, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(65, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(65, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(114, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(65, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(65, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_fractial_input_3) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetPosition(root, CSSEdgeTop, 0.3f);
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 113.4f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 50);
  CSSNodeStyleSetHeight(root_child0, 20);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeStyleSetHeight(root_child2, 10);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(114, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(114, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}

TEST(CSSLayoutTest, rounding_fractial_input_4) {
  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, true);

  const CSSNodeRef root = CSSNodeNew();
  CSSNodeStyleSetPosition(root, CSSEdgeTop, 0.7f);
  CSSNodeStyleSetWidth(root, 100);
  CSSNodeStyleSetHeight(root, 113.4f);

  const CSSNodeRef root_child0 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child0, 1);
  CSSNodeStyleSetFlexBasis(root_child0, 50);
  CSSNodeStyleSetHeight(root_child0, 20);
  CSSNodeInsertChild(root, root_child0, 0);

  const CSSNodeRef root_child1 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child1, 1);
  CSSNodeStyleSetHeight(root_child1, 10);
  CSSNodeInsertChild(root, root_child1, 1);

  const CSSNodeRef root_child2 = CSSNodeNew();
  CSSNodeStyleSetFlexGrow(root_child2, 1);
  CSSNodeStyleSetHeight(root_child2, 10);
  CSSNodeInsertChild(root, root_child2, 2);
  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionLTR);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(1, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeCalculateLayout(root, CSSUndefined, CSSUndefined, CSSDirectionRTL);

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root));
  ASSERT_FLOAT_EQ(1, CSSNodeLayoutGetTop(root));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root));
  ASSERT_FLOAT_EQ(113, CSSNodeLayoutGetHeight(root));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child0));
  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetTop(root_child0));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child0));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetHeight(root_child0));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child1));
  ASSERT_FLOAT_EQ(64, CSSNodeLayoutGetTop(root_child1));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child1));
  ASSERT_FLOAT_EQ(25, CSSNodeLayoutGetHeight(root_child1));

  ASSERT_FLOAT_EQ(0, CSSNodeLayoutGetLeft(root_child2));
  ASSERT_FLOAT_EQ(89, CSSNodeLayoutGetTop(root_child2));
  ASSERT_FLOAT_EQ(100, CSSNodeLayoutGetWidth(root_child2));
  ASSERT_FLOAT_EQ(24, CSSNodeLayoutGetHeight(root_child2));

  CSSNodeFreeRecursive(root);

  CSSLayoutSetExperimentalFeatureEnabled(CSSExperimentalFeatureRounding, false);
}
